#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

# Module for the compartment

package Compartment;
use v5.014;
use strict;
use warnings;
use Carp;


use lib '.';
use parent 'Object';


my $float_patt = '\d+\.?(\d*)([eE][+-]?\d+)?';


# Create new compartment, set volume to 1 l as the default

sub new {
	shift->SUPER::new(volume    => 1,
					  thickness => 0);
}


# Sets volume of comparment

sub set_volume {
	my ($self, $vol) = @_;
	croak "volume is not a number\n" unless $vol =~ /^\s*($float_patt)\s*$/;
	$self->{volume} = $1;
}


# Sets thickness of membrane

sub set_thickness {
	my ($self, $t) = @_;
	croak "thickness is not a number\n" unless $t =~ /^\s*($float_patt)\s*$/;
	$self->{thickness} = $1;
}


# Return string with COPASI XML code for compartment

sub emit {
	my ($self, $indent) = @_;
	my $s = "$indent<ListOfCompartments>\n";
	$indent .= '  ';
	$s .=   "$indent<Compartment key=\"Compartment_0\" "
		  . "name=\"compartment\" "
		  . "simulationType=\"fixed\" "
		  . "dimensionality=\"3\" "
		  . "addNoise=\"false\"/>\n";
	$indent =~ s/  //;
	$s .= "$indent</ListOfCompartments>\n";
}


1;
