#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

# Module for constants

package Const;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';
use Range;

my $float_patt = '\d+\.?(\d*)([eE][+-]?\d+)?';


sub new {
	my ($class, $state, $desc) = @_;
	my @args = split /\s*=\s*/, $desc;

	croak "invalid constant name '$args[0]'\n"
		unless $args[0] =~ /^[a-z_][a-z0-9_]*/i;

	my $self = $class->SUPER::new(state => $state,
								  name  => $args[0],
								  desc  => $args[0]);

	for (@{$state->consts->list}) {
		croak "constant '$args[0]' set twice\n" if $args[0] eq $_->name;
	}

	bless $self, $class;

	croak "invalid constant '$desc'\n" unless defined $args[1];
	
	if ($args[1] =~ /^($float_patt)$/) {
		$self->{value} = $args[1];
	} elsif ($args[1] =~ /^\[([^\]]+)\]$/) {
		$self->{value} = Range->new($self, $1);
	} else {
		croak "invalid constant '$desc'\n";
	}

	$self->state->consts->add($self);
}


sub value {
	my $self = shift;
	return ref($self->{value}) eq 'Range' ?
		   $self->{value}->value : $self->{value};
}


1;
