#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Func_Param;
use v5.014;
use strict;
use warnings;

use lib '.';
use parent 'Object';


sub new {
	my ($class, $state, $function, $name) = @_;

	my $type = 'constant';
	if ($name =~ /^sub/i) {
		$type = 'substrate';
	} elsif ($name =~ /^prod/i) {
		$type = 'product';
	}

	my $order = scalar @{$state->func_params->all_of_function($function)};

	my $self = $class->SUPER::new(state    => $state,
								  function => $function,
								  name     => $name,
								  type     => $type,
								  order    => $order);

	$self->state->func_params->add($self);
}


sub emit {
	my ($self, $indent) = @_;
	sprintf   "%s<ParameterDescription key=\"FunctionParameter_%d\" "
	        . "name=\"%s\" "
			. "order=\"%d\" "
			. "role=\"%s\"/>\n",
			$indent, $self->state->func_params->index($self),
			$self->name, $self->order, $self->type;
}


1;
