#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

# Module for list of all functions (mass action functions are
# added automatically, the rest must be defined by the user)

package Function_List;
use v5.014;
use strict;
use warnings;

use lib '.';
use parent 'List';
use Function;


# Creates instance of the function list

sub new {
	my $self = shift->SUPER::new('Function');
	$self->{state} = shift;
	return $self;
}


sub init {
	my $self = shift;
	Function->new($self->{state},
				    'Mass action (irreversible): k1*PRODUCT&lt;substrate_i>, '
				  . 'irrev, MassAction');
	Function->new($self->{state},
				    'Mass action (reversible): '
				  . 'k1*PRODUCT&lt;substrate_i>+k2*PRODUCT&lt;product_j>, '
				  . 'rev, MassAction');

}


# Returns a string with the COPASI XML for all functions in the list

sub emit {
	my ($self, $indent) = @_;
	my $s = "$indent<ListOfFunctions>\n";
	$indent .= '  ';
	$s .= $_->emit($indent) for @{$self->list};
	$indent =~ s/  //;
	$s .= "$indent</ListOfFunctions>\n";
}


1;
