#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Iterator;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';


# Object for iterating over the value arrays of one or more Range
# objects that have been added to the iterator.

sub new {
	my $self = shift->SUPER::new(list  => [],
								 index => []);
}


# Add  new 'Range' object (mostly a wrapper for an array) to iterator list

sub add {
	my ($self, $obj) = @_;
	croak "'$obj' not an iteratable object\n" unless ref($obj) eq 'Range';
	croak "'$obj' is already in the iterator list\n"
		if !! grep {$_ == $obj} @{$self->list};
	croak "iterator already in use, can't add more items\n"
		if grep {$_ != 0} @{$self->index};

	push @{$self->list}, $obj;
	push @{$self->index}, 0;
}


# Return the current index into the Ranges array of values

sub current_index {
	my ($self, $inv) = @_;
	my $i = 0;
	for (@{$self->list}) {
		return $self->index->[$i] if $_ == $inv;
		$i++;
	}
	return undef;
}


# Sets the iterator up for the next iteration step. Returns the iterator
# object or 'undef' when the iterator has reached the end of the iterations

sub next {
	my $self = shift;
	return undef unless @{$self->list};

	my $i = @{$self->index} - 1;
	for (reverse @{$self->list}) {
		return $self if ++$self->index->[$i] < $_->count;
		return undef if $i == 0;
		$self->index->[$i--] = 0;
	}
}


# Resets the iterator to its start state

sub reset {
	$_ = 0 for @{shift->index};
}


sub desc {
	my $self = shift;
	return '' unless @{$self->list};
	return "_" . join ',', map {$_->desc} @{$self->list};
}


1;
