#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

# Parent class for all lists

package List;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';


# Create new list, first argument ist the type of the list, all following
# are the names of classes of objects that can be added to the instance of
# the list

sub new {
	my $self = shift->SUPER::new(list   => [],
								 accept => [@_]);
}


# Add new object to the list, nake sure it has an acceptable type
# and isn't already in the list

sub add {
	my ($self, $obj) = @_;
	for (@{$self->accept}) {
		next unless ref($obj) eq $_;
		croak "'$obj' already in list\n" if $self->exists($obj);
		push @{$self->list}, $obj;
		return $obj;
	}
	croak "invalid type of object\n";
}


# Return number of elements of the list

sub count {
	scalar @{shift->list};
}


# Return if an object is in the list

sub exists {
	my ($self, $obj) = @_;
	!! grep {$_ == $obj} @{$self->list};
}


# Return the index of the given object in the list

sub index {
	my ($self, $item) = @_;
	my $i = 0;
	for (@{$self->list}) {
		return $i if $_ == $item;
		$i++;
	}

	return undef;
}


# Return the object with the given name

sub get {
	my ($self, $name) = @_;
	for (@{$self->list}) {
		return $_ if $_->name eq $name;
	}
	return undef;
}


1;
