#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Model;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';


sub new {
	my $self = shift->SUPER::new(state => shift,
								 name  => 'Model');
}


sub set_name {
	my ($self, $name) = @_;
	croak "missing name\n" if not  defined $name || $name =~ /^\s*$/;
	$self->{name} = $name;
}


sub emit {
	my ($self, $indent) = @_;
	my $s = sprintf   "%s<Model key=\"Model_0\" "
		            . "name=\"%s\" "
					. "simulationType=\"time\" "
					. "timeUnit=\"s\" "
					. "volumeUnit=\"l\" "
					. "areaUnit=\"m²\" "
					. "lengthUnit=\"m\" "
					. "quantityUnit=\"mol\" "
					. "type=\"deterministic\" "
					. "avogadroConstant=\"6.022140857e+23\">\n",
					$indent, $self->name;
	$indent .= '  ';
	$s .= $self->state->compartment->emit($indent);
	$s .= $self->state->species->emit($indent);
	$s .= $self->state->reactions->emit($indent);
	$s .= $self->emit_params($indent);
	$indent =~ s/  //;
	$s .=  "$indent</Model>\n";
}


sub emit_params {
	my ($self, $indent) = @_;
	my $s =   "$indent<ListOfModelParameterSets "
		    . "activeSet=\"ModelParameterSet_0\">\n";

	$indent .= '  ';
	$s .=   "$indent<ModelParameterSet key=\"ModelParameterSet_0\" "
		  . "name=\"Initial State\">\n";

	$indent .= '  ';
	$s .=   "$indent<ModelParameterGroup cn=\"String=Initial Time\" "
		  . "type=\"Group\">\n";
	$indent .= '  ';
	$s .= sprintf   "%s<ModelParameter cn=\"CN=Root,Model=%s\" "
		          . "value=\"0\" type=\"Model\" simulationType=\"time\"/>\n",
		          $indent, $self->name;
	$indent =~ s/  //;
	$s .= "$indent</ModelParameterGroup>\n";
	$s .=   "$indent<ModelParameterGroup cn=\"String=Initial Compartment "
		  . "Sizes\" type=\"Group\">\n";

	$indent .= '  ';
	$s .= sprintf  "%s<ModelParameter "
		         . "cn=\"CN=Root,Model=%s,Vector=Compartments[compartment]\" "
				 . "value=\"%.6g\" "
				 . "type=\"Compartment\" "
				 . "simulationType=\"fixed\"/>\n",
				 $indent, $self->name, $self->state->compartment->volume;
	$indent =~ s/  //;
	$s .= "$indent</ModelParameterGroup>\n";

	$s .=   "$indent<ModelParameterGroup "
		  . "cn=\"String=Initial Species Values\" type=\"Group\">\n";
	$indent .= '  ';

	for (@{$self->state->species->all_used}) {
		$s .= sprintf   "%s<ModelParameter "
			          . "cn=\"CN=Root,Model=%s,"
					  . "Vector=Compartments[compartment], "
					  . "Vector=Metabolites[%s]\" "
					  . "value=\"%.6g\" "
					  . "type=\"Species\" "
					  . "simulationType=\"%s\"/>\n",
					  $indent, $self->name, $_->name, $_->particle_number,
					  $_->type;
	}
	
	$indent =~ s/  //;
	$s .= "$indent</ModelParameterGroup>\n";

	$s .=   "$indent<ModelParameterGroup cn=\"String=Kinetic Parameters\" "
		  . "type=\"Group\">\n";

	$indent .= '  ';
	for my $r (@{$self->state->reactions->list}) {
		$s .= sprintf   "%s<ModelParameterGroup "
			          . "cn=\"CN=Root,Model=%s,Vector=Reactions[%s]\" "
					  . "type=\"Reaction\">\n",
					  $indent, $self->name, $r->name;
		$indent .= '  ';
		for my $c (@{$r->constants}) {
			$s .= sprintf   "%s<ModelParameter "
				          . "cn=\"CN=Root,Model=%s,Vector=Reactions[%s],"
						  . "ParameterGroup=Parameters,Parameter=%s\" "
						  . "value=\"%.6g\" "
						  . "type=\"ReactionParameter\" "
						  . "simulationType=\"fixed\"/>\n",
						  $indent, $self->name, $r->name, $c->{name},
						  $c->value;
		}

		$indent =~ s/  //;
		$s .= "$indent</ModelParameterGroup>\n";
	}

	$indent =~ s/  //;
	$s .= "$indent</ModelParameterGroup>\n";

	$indent =~ s/  //;
	$s .= "$indent</ModelParameterSet>\n";

	$indent =~ s/  //;
    $s .= "$indent</ListOfModelParameterSets>\n";

	$s .= "$indent<StateTemplate>\n";
	$indent .= '  ';

	$s .= "$indent<StateTemplateVariable objectReference=\"Model_0\"/>\n";
	my $v = "0";
	for (@{$self->state->species->all_used}) {
		$s .= sprintf   "%s<StateTemplateVariable "
			          . "objectReference=\"Metabolite_%s\"/>\n",
			          $indent, $self->state->species->index($_);
		$v .= sprintf " %.6g", $_->particle_number;
	}
	$s .= "$indent<StateTemplateVariable objectReference=\"Compartment_0\"/>\n";
	$v .= sprintf " %.6g", $self->state->compartment->volume;

	$indent =~ s/  //;
    $s .= "$indent</StateTemplate>\n";
	$s .= "$indent<InitialState type=\"initialState\">\n";
	$indent .= '  ';
	$s .= "$indent$v\n";
	$indent =~ s/  //;
	$s .= "$indent</InitialState>\n";
}


1;
