#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Param;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';
use Range;
use Expr;

my $float_patt = '\d+\.?(\d*)([eE][+-]?\d+)?';


sub new {
	my ($class, $reaction, $desc) = @_;
	my @args = split /\s*=\s*/, $desc;

	my $self = $class->SUPER::new(state    => $reaction->state,
								  reaction => $reaction,
								  name     => $args[0],
								  desc     =>   $args[0]
								              . '(' . $reaction->name . ')');

	for (@{$reaction->state->params->all_of_reaction($reaction)}) {
		croak "parameter '$args[0]' set twice\n" if $args[0] eq $_->name;
	}

	bless $self, $class;

	if ($args[1] =~ /^($float_patt)$/) {
		$self->{value} = $args[1];
	} elsif ($args[1] =~ /^\[([^\]]+)\]$/) {
		$self->{value} = Range->new($self, $1);
	} else {
		$self->{value} = Expr->new($self->{state}, $args[1]);
	}

	$self->state->params->add($self);
}


sub value {
	my $self = shift;
	return ref($self->{value}) ? $self->{value}->value : $self->{value};
}


sub emit {
	my ($self, $indent) = @_;
	sprintf   "%s<Constant key=\"Parameter_%d\" "
		    . "name=\"%s\" "
			. "value=\"%.6g\"/>\n",
			$indent, $self->state->params->index($self),
			$self->name, $self->value;
}

1;
