#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Param_Base;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object;


sub new {
	my ($class, $reaction, $name) = @_;

	for (@{$reaction->state->params->all_of_reaction($reaction)}) {
		croak   "parameter '$name' of reaction ' . $reaction->name
              . 'set twice\n" if $name eq $_->name;
	}

	my $self = $class->SUPER::new(state    => $reaction->state,
								  reaction => $reaction,
								  name     => $name);
	$self->state->params->add($self);

	return $self;
}


# Return string with COPASI XML code for the parameter

sub emit {
	my ($self, $indent) = @_;
	sprintf   "%s<Constant key=\"Parameter_%d\" "
	        . "name=\"%s\" "
	        . "value=\"%.6g\"/>\n",
	        $indent, $self->state->params->index($self),
	        $self->name, $self->value;
}


1;
