#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

# Package for list of all plots

package Plot_List;
use v5.014;
use strict;
use warnings;

use lib '.';
use parent 'List';
use Plot;


# Creates instance of the plot list

sub new {
	my $self = shift->SUPER::new('Plot');
	$self->{state} = shift;
	return $self;
}


sub add_plot {
	my ($self, $line) = @_;

	if ($line =~ /^\s*((Concentration|Rate))s\s*:\s*(.*?)\s*?$/) {
		my $type = $1;
		my $tmp = $3;
		my ($log_x_scale, $log_y_scale) = 0;
		if ($tmp =~ /\s*\[\s*log_?y\s*\]\s*$/) {
			$log_y_scale = 1;
			$tmp =~ s/\s*\[\s*log_?y\s*\]\s*$//;
		}
		if ($tmp =~ /\s*\[\s*log_?x\s*\]\s*$/) {
			$log_x_scale = 1;
			$tmp =~ s/\s*\[\s*log_?x\s*\]\s*$//;
		}
		if ($tmp =~ /\s*\[\s*log_?y\s*\]\s*$/) {
			$log_y_scale = 1;
			$tmp =~ s/\s*\[\s*log_?y\s*\]\s*$//;
		}
		$tmp =~ s/\s*$//;

		Plot->new($self->{state}, $type, $tmp, $log_x_scale, $log_y_scale);
		return;
	}

	die "Unrecognized input\n" unless @{$self->list};

	$self->list->[-1]->add_species($line);
}


sub check {
	my ($self,$ with_rates) = @_;

	# Check if user asked for concentration and rate plots

	my ($has_conc_plot, $has_rates_plot) = (0, 0);

	for (@{$self->list}) {
		$has_conc_plot = $_->type eq 'Concentration';
		$has_rates_plot = $_->type eq 'Rate';
	}

	# Make sure rates are written to the report file if a rate plot was
	# requested

	$self->{state}->report->set_with_rates(1)
		if $has_rates_plot && ! $with_rates;

	# Make sure a concentrarion and rate plot gets created if the user
	# didn't explicitely asked for plots

	Plot->new($self->{state}, 'Concentration', 'Concentrations')
		unless $has_conc_plot;
	Plot->new($self->{state}, 'Rate', 'Rates')
		if ! $has_rates_plot && $with_rates;
}			  
		

sub emit {
	my ($self, $indent) = @_;

	return '' unless @{$self->list};

	my $s = "$indent<ListOfPlots>\n";
	$indent .= '  ';

	$s .= $_->emit($indent) for @{$self->list};

	$indent =~ s/  //;
	return $s .= "$indent</ListOfPlots>\n";
}


sub gnuplot_plot {
	my $self = shift;
	$_->gnuplot_plot(@_) for @{$self->list};
}


1;
