#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Species;
use v5.014;
use strict;
use warnings;

use lib '.';
use parent 'Species_Base';
use Species_Assignment;

my $float_patt = '\d+\.?(\d*)([eE][+-]?\d+)?';


sub new {
	my ($class, $state, $str) = @_;
	my @args = split /\s*,\s*/, $str;
	my $fixed =    $args[1] =~ /\s+fixed$/
		        || (@args >=3 && $args[2] eq 'fixed');
	my $self;
	if (   $args[1] =~ /^($float_patt)$/
		|| $args[1] =~ /^($float_patt)\s+fixed$/) {
		$self = $class->SUPER::new($state, $args[0],
								   $fixed ? 'fixed' : 'reaction');
		$self->{conc} = $1;
	} elsif ($args[1] =~ /^\[([^\]]+)\]$/) {
		$self = $class->SUPER::new($state, $args[0],
								   $fixed ? 'fixed' : 'reaction');
		$self->{conc} = Range->new($self, $1);
	} else {
		return Species_Assignment->new($state, @args);
	}

	return $self;
}


# Return concentration of spcies

sub concentration {
	my $self = shift;
	return ref($self->{conc}) ? $self->{conc}->value : $self->{conc};
}


# Return name of species to be used in file names

sub desc {
	'[' . shift->name . ']';
}


# Return string with COPASI XML code for the species

sub emit {
	my ($self, $indent) = @_;
	sprintf   "%s<Metabolite key=\"Metabolite_%d\" "
		    . "name=\"%s\" "
            . "simulationType=\"%s\" "
		    . "compartment=\"Compartment_0\" "
		    . "addNoise=\"false\"/>\n",
		    $indent, $self->state->species->index($self),
			$self->name, $self->type;
}


1;
