#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Species_Assignment;
use v5.014;
use strict;
use warnings;
use Carp;
use POSIX;
use Math::Trig;

use lib '.';
use parent 'Species_Base';


my %predefined = (abs          => 'abs',
				  root         => 'sqrt',
				  floor        => 'floor',
				  ceil         => 'ceil',
				  log          => 'log',
				  log10        => 'log10',
				  exp          => 'exp',
				  sin          => 'sin',
				  cos          => 'cos',
				  tan          => 'tan',
				  sec          => 'sec',
				  csc          => 'csc',
				  cot          => 'cotan',
				  sinh         => 'sinh',
				  cosh         => 'cosh',
				  tanh         => 'tanh',
				  sech         => 'sech',
				  csch         => 'csch',
				  coth         => 'cotanh',
				  asin         => 'asin',
				  acos         => 'acos',
				  atan         => 'atan',
				  arcsec       => 'asec',
				  arccsc       => 'acsc',
				  arccot       => 'acotan',
				  arcsinh      => 'asinh',
				  arccosh      => 'acosh',
				  arctanh      => 'atanh',
				  arcsech      => 'asech',
				  arccsch      => 'acsch',
				  arccoth      => 'acotanh',
				  pi           => 'pi',
				  exponentiale => 'exp(1)');


sub new {
	my ($class, $state, $name, $ass) = @_;
	my $self = $class->SUPER::new($state, $name, 'assignment');
	$self->{assignment} = $ass;
	$self->_check_assignment($self->{assignment});
	return $self;
}


sub concentration {
	my $self = shift;
	my $a = $self->{assignment};
	$a =~ s/\[([^\]]+)\](_0)?/$self->state->species->get($1)->concentration/eg;
	$a =~ s/(^|\W)$_(\W|$)/$1$predefined{$_}$2/gi for keys %predefined;
	$a =~ s/\^/**/g;
	eval $a;
}


sub emit {
	my ($self, $indent) = @_;
	my $s = sprintf   "%s<Metabolite key=\"Metabolite_%d\" "
		            . "name=\"%s\" "
					. "simulationType=\"assignment\" "
					. "compartment=\"Compartment_0\" "
		            . "addNoise=\"false\">\n",
					$indent, $self->state->species->index($self),
					$self->name;

	$indent .= '  ';
	$s .= "$indent<Expression>\n";
	$indent .= '  ';

	my $a = $self->{assignment};
	my $rep = sprintf   '&lt;CN=Root,Model=%s,Vector=Compartments[compartment],'
		              . 'Vector=Metabolites[',
		              $self->{state}->model->name;
	$a =~ s/\[([^\]]+)\](?!_0)/$rep$1],Reference=Concentration>/g;
	$a =~ s/\[([^\]]+)\]_0/$rep$1],Reference=InitialConcentration>/g;
	$s .= "$indent$a\n";
	$indent =~ s/  //;
	$s .= "$indent</Expression>\n";
	$indent =~ s/  //;
	$s .= "$indent</Metabolite>\n";
}


sub _check_assignment {
	my ($self, $ass) = @_;
	my $a = $ass;
	while ($a =~ /\[([^\]]+)\](_0)?/) {
		my $s = $self->state->species->get($1);
		croak   "species named '$1' is not defined but used in assignment "
			  . "'$ass'\n" unless $s;
		$a =~ s/\[([^\]]+)\](_0)?/$s->concentration/e;
	}
	$a =~ s/(^|\W)$_(\W|$)/$1$predefined{$_}$2/gi for keys %predefined;
	croak "Invalid formula '$ass'\n" unless defined eval $a;
	return $ass;
}
	
1;
