#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Species_Base;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';

my $avo = 6.022140857e23;


sub new {
	my ($class, $state, $name, $type) = @_;
	my $self = $class->SUPER::new(state   => $state,
								  name    => $name,
								  type    => $type,
								  is_used => 0);
	$self->state->species->add($self);
}


# Return number of particles of the species

sub particle_number {
	my $self = shift;
	return $avo * $self->state->compartment->volume * $self->concentration;
}


# Return if species is part of a reaction 

sub is_used {
	my $self = shift;
	$self->{is_used} = $_[0] ? 1 : 0 if defined $_[0];
	return $self->{is_used};
}


1;
