#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

# Package for the list of species found in the input

package Species_List;
use v5.014;
use strict;
use warnings;

use lib '.';
use parent 'List';


# Create list of species

sub new {
	shift->SUPER::new('Species', 'Species_Assignment');
}


# Return array reference to list of all species used in reactions

sub all_used {
	[grep {$_->is_used} @{shift->list}];
}


sub find {
	my ($self, $name) = @_;
	$name =~ s/^\[([^\]]+)\]$/$1/;
	for (@{$self->list}) {
		return $_ if $_->name eq $name;
	}
	return undef;
}


# Return string with COPASI XML code for all species

sub emit {
	my ($self, $indent) = @_;
	my $s = "$indent<ListOfMetabolites>\n";
	$indent .= '  ';
	$s .= $_->emit($indent) for @{$self->list};
	$indent =~ s/  //;
	$s .= "$indent</ListOfMetabolites>\n";
}


1;
