#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package State;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';

use Model;
use Compartment;
use Const_List;
use Function_List;
use Func_Param_List;
use Species_List;
use Species;
use Reaction_List;
use Reaction;
use Param_List;
use Iterator;
use Task;
use Report;
use Plot_List;


sub new {
	my $self = {};
	bless $self, shift;
	
	$self->{model}       = Model->new($self);
	$self->{compartment} = Compartment->new;
	$self->{consts}      = Const_List->new($self);
	$self->{functions}   = Function_List->new($self);
	$self->{func_params} = Func_Param_List->new;
	$self->{species}     = Species_List->new;
	$self->{reactions}   = Reaction_List->new;
	$self->{params}      = Param_List->new;
	$self->{iterator}    = Iterator->new;
	$self->{task}        = Task->new($self);
	$self->{report}      = Report->new($self);
	$self->{plots}       = Plot_List->new($self);
	
	# Create accessor methods for all the above attributes

	no strict 'refs';
	for my $attr (keys %{$self}) {
		*{__PACKAGE__ . '::' . $attr} = sub {shift->{$attr}};
	}

	$self->functions->init;
	$self->consts->init;

	return $self;
}


sub add_constant {
	Const->new(@_);
}


sub add_function {
	Function->new(@_);
}


sub add_species {
	Species->new(@_);
}


sub add_reaction {
	Reaction->new(@_);
}


sub emit {
	my $self = shift;
	my $s =   "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
		    . "<COPASI xmlns=\"http://www.copasi.org/static/schema\">\n";
	my $indent = '  ';

	$s .= $self->functions->emit($indent);
	$s .= $self->model->emit($indent);
	$s .= $self->task->emit($indent);
	$s .= $self->report->emit($indent);
	$s .= $self->plots->emit($indent);
	$s .= "</COPASI>\n";
}


1;
