#  This file is part of the copd package
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
# 
#  The program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#  Copyright (C) 2019 Jens Thoms Törring <jt@toerring.de>
#

package Task;
use v5.014;
use strict;
use warnings;
use Carp;

use lib '.';
use parent 'Object';

my $float_patt = '\d+\.?(\d*)([eE][+-]?\d+)?';


sub new {
	shift->SUPER::new(state    => shift,
					  duration => 1);
}


sub set_duration {
	my ($self, $duration) = @_;
	croak "task duration '$_[0]' not a number\n"
		unless $duration =~ /^($float_patt)$/;
	$self->{duration} = $1;
}

sub emit {
	my ($self, $indent) = @_;
	my $s = "$indent<ListOfTasks>\n";
	$indent .= '  ';

	$s .=   "$indent<Task key=\"Task_0\" name=\"Time-Course\" "
		  . "type=\"timeCourse\" scheduled=\"true\" updateModel=\"false\">\n";

	$indent .= '  ';
	$s .= sprintf   "%s<Report "
		          . "reference=\"Report_0\" "
		          . "target=\"%s\" "
				  . "append=\"0\" "
				  . "confirmOverwrite=\"0\"/>\n",
				  $indent, $self->state->report->file_name . '.crp';
	$s .= "$indent<Problem>\n";
	$indent .= '  ';

	$s .=   "$indent<Parameter name=\"AutomaticStepSize\" "
		  . "type=\"bool\" value=\"1\"/>\n";
    $s .=   "$indent<Parameter name=\"StepNumber\" "
		  . "type=\"unsignedInteger\" value=\"10000\"/>\n";
    $s .= sprintf   "%s<Parameter name=\"StepSize\" "
	              . "type=\"float\" value=\"%.6g\"/>\n",
		          $indent, $self->duration / 10000;
	$s .= sprintf   "%s<Parameter name=\"Duration\" "
		          . "type=\"float\" value=\"%.6g\"/>\n",
		          $indent, $self->duration;
	$s .=   "$indent<Parameter name=\"TimeSeriesRequested\" "
		  . "type=\"bool\" value=\"1\"/>\n";
	$s .=   "$indent<Parameter name=\"OutputStartTime\" "
		  . "type=\"float\" value=\"0\"/>\n";
	$s .=   "$indent<Parameter name=\"Output Event\" "
		  . "type=\"bool\" value=\"0\"/>\n";
	$s .=   "$indent<Parameter name=\"Start in Steady State\" "
		  . "type=\"bool\" value=\"0\"/>\n";

	$indent =~ s/  //;
	$s .= "$indent</Problem>\n";

	$s .=   "$indent<Method name=\"Deterministic (LSODA)\" "
		  . "type=\"Deterministic(LSODA)\">\n";

	$indent .= '  ';
	$s .=   "$indent<Parameter name=\"Integrate Reduced Model\" "
		  . "type=\"bool\" value=\"0\"/>\n";
	$s .=   "$indent<Parameter name=\"Relative Tolerance\" "
		  . "type=\"unsignedFloat\" value=\"1e-06\"/>\n";
	$s .=   "$indent<Parameter name=\"Absolute Tolerance\" "
		  . "type=\"unsignedFloat\" value=\"1e-12\"/>\n";
	$s .=   "$indent<Parameter name=\"Max Internal Steps\" "
		  . "type=\"unsignedInteger\" value=\"1000000\"/>\n";
	$s .=   "$indent<Parameter name=\"Max Internal Step Size\" "
		  . "type=\"unsignedFloat\" value=\"0\"/>\n";

	$indent =~ s/  //;
	$s .= "$indent</Method>\n";

	$indent =~ s/  //;
	$s .= "$indent</Task>\n";

	$indent =~ s/  //;
	$s . "$indent</ListOfTasks>\n";
}

1;
